/*
 * AIEngine a new generation network intrusion detection system.
 *
 * Copyright (C) 2013-2020  Luis Campo Giralte
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 51 Franklin St, Fifth Floor,
 * Boston, MA  02110-1301, USA.
 *
 * Written by Luis Campo Giralte <luis.camp0.2009@gmail.com> 2013
 *
 */
#ifndef _TEST_TCPGENERIC_H_
#define _TEST_TCPGENERIC_H_

#include <string>
#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include "Protocol.h"
#include "StackTest.h"
#include "../ip/IPProtocol.h"
#include "../ip6/IPv6Protocol.h"
#include "../tcp/TCPProtocol.h"
#include "TCPGenericProtocol.h"

using namespace aiengine;

struct StackTCPGenericTest : public StackTest
{
        IPProtocolPtr ip;
        IPv6ProtocolPtr ip6;
        TCPProtocolPtr tcp;
        TCPGenericProtocolPtr gtcp;
        TCPProtocolPtr tcp6;
        TCPGenericProtocolPtr gtcp6;
        MultiplexerPtr mux_ip;
        MultiplexerPtr mux_ip6;
        MultiplexerPtr mux_tcp;
        MultiplexerPtr mux_tcp6;

       // FlowManager and FlowCache
        FlowManagerPtr flow_mng;
        FlowCachePtr flow_cache;

        // FlowForwarders
        SharedPointer<FlowForwarder> ff_tcp;
        SharedPointer<FlowForwarder> ff_gtcp;
        SharedPointer<FlowForwarder> ff_tcp6;
        SharedPointer<FlowForwarder> ff_gtcp6;

        StackTCPGenericTest()
        {
                ip = IPProtocolPtr(new IPProtocol());
                ip6 = IPv6ProtocolPtr(new IPv6Protocol());
                tcp = TCPProtocolPtr(new TCPProtocol());
                gtcp = TCPGenericProtocolPtr(new TCPGenericProtocol());
                tcp6 = TCPProtocolPtr(new TCPProtocol());
                gtcp6 = TCPGenericProtocolPtr(new TCPGenericProtocol());
                mux_ip = MultiplexerPtr(new Multiplexer());
                mux_ip6 = MultiplexerPtr(new Multiplexer());
                mux_tcp = MultiplexerPtr(new Multiplexer());
                mux_tcp6 = MultiplexerPtr(new Multiplexer());
                mux_tcp6 = MultiplexerPtr(new Multiplexer());
                ff_tcp = SharedPointer<FlowForwarder>(new FlowForwarder());
                ff_gtcp = SharedPointer<FlowForwarder>(new FlowForwarder());
                ff_tcp6 = SharedPointer<FlowForwarder>(new FlowForwarder());
                ff_gtcp6 = SharedPointer<FlowForwarder>(new FlowForwarder());

                // Allocate the flow caches and tables
                flow_mng = FlowManagerPtr(new FlowManager());
                flow_cache = FlowCachePtr(new FlowCache());

                // configure the ip
                ip->setMultiplexer(mux_ip);
                mux_ip->setProtocol(static_cast<ProtocolPtr>(ip));

                // configure the ip6
                ip6->setMultiplexer(mux_ip6);
                mux_ip6->setProtocol(static_cast<ProtocolPtr>(ip6));

                // configure the tcp 
                tcp->setMultiplexer(mux_tcp);
                mux_tcp->setProtocol(static_cast<ProtocolPtr>(tcp));

                // configure the generic tcp 
                gtcp->setFlowForwarder(ff_gtcp);
                ff_gtcp->setProtocol(static_cast<ProtocolPtr>(gtcp));

                // configure the tcp for ipv6
                tcp6->setMultiplexer(mux_tcp6);
                mux_tcp6->setProtocol(static_cast<ProtocolPtr>(tcp6));

                // configure the generic tcp for ipv6
                gtcp6->setFlowForwarder(ff_gtcp6);
                ff_gtcp6->setProtocol(static_cast<ProtocolPtr>(gtcp6));

                mux_eth->addUpMultiplexer(mux_ip);
                mux_eth->addUpMultiplexer(mux_ip6);
                
		mux_ip->addDownMultiplexer(mux_eth);
                mux_ip->addUpMultiplexer(mux_tcp);
                
		mux_ip6->addDownMultiplexer(mux_eth);
                mux_ip6->addUpMultiplexer(mux_tcp6);
                
		mux_tcp->addDownMultiplexer(mux_ip);
                mux_tcp6->addDownMultiplexer(mux_ip6);

                // Connect the FlowManager and FlowCache
                flow_cache->createFlows(1);
		tcp->createTCPInfos(1);
		tcp6->createTCPInfos(1);

                tcp->setFlowCache(flow_cache);
                tcp->setFlowManager(flow_mng);
                tcp6->setFlowCache(flow_cache);
                tcp6->setFlowManager(flow_mng);

                // Configure the FlowForwarders
                tcp->setFlowForwarder(ff_tcp);
                ff_tcp->addUpFlowForwarder(ff_gtcp);

                tcp6->setFlowForwarder(ff_tcp6);
                ff_tcp6->addUpFlowForwarder(ff_gtcp6);
        }

        ~StackTCPGenericTest() {}
};

#endif
